/***********************************************************************************
 * mp3info.h
 * Ryan Gilfether <hotrodder@rocketmail.com>
 * Apr 25, 2001
 *
 * Reads the first four bytes of an mp3 file and extracts all header information
 * View the README file included in the distribution for more information
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA (or visit
 * their web site at http://www.gnu.org/).
 ***********************************************************************************/

#ifndef MP3INFO_H
#define MP3INFO_H

#include <iostream>
#include <fstream>
#include <map>

#define EXTRACT_BITS(the_val, bits_start, bits_len) ((the_val >> (bits_start - 1)) & ((1 << bits_len) - 1))
#define DWORD unsigned long

class mp3info
{
	public:
		mp3info(char* name);
                mp3info(ifstream* f);
		~mp3info();
		char* emphasis();
		int length();
		float version();
		int layer();
		int bitrate();
		int frames();
		bool is_private();
		char* mode();
		bool copyright();
		int frequency();
                bool is_protected();
                bool original();
		long size();

	private:
		ifstream* file;
                bool opened_file;
		long _size;
		map<char*, int,less<char*> > bits;
		char szbuff[30];
		void initialize();
		long get_size();
};


const int _bitrates[2][3][16] =
{
	// mpeg 2.0 and 2.5
	{
		{0,  8, 16, 24, 32, 40, 48,  56,  64,  80,  96, 112, 128, 144, 160, 0}, // layer 3
		{0,  8, 16, 24, 32, 40, 48,  56,  64,  80,  96, 112, 128, 144, 160, 0}, // layer 2
		{0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256, 0}  // layer 1
	},

	// mpeg 1.0
	{
		{0, 32, 40, 48,  56,  64,  80,  96, 112, 128, 160, 192, 224, 256, 320, 0}, // layer 3
		{0, 32, 48, 56,  64,  80,  96, 112, 128, 160, 192, 224, 256, 320, 384, 0}, // layer 2
		{0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448, 0}  // layer 1
	}
};

const int _frequency[4][3] =
{
	{32000, 16000,  8000}, // mpeg 2.5
	{    0,     0,     0}, // not used
	{22050, 24000, 16000}, // mpeg 2.0
	{44100, 48000, 32000}  // mpeg 1.0
};

const float _versions[4] =
{
        2.5, 0.0, 2.0, 1.0
};

#endif